@@ -21,9 +21,13 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GDK_H_INSIDE__) && !defined (GDK_COMPILATION)
+#error "Only <gdk/gdk.h> can be included directly."
+#endif
+
 #ifndef __GDK_WINDOW_H__
 #define __GDK_WINDOW_H__
 
@@ -33,9 +37,10 @@
 
 G_BEGIN_DECLS
 
-typedef struct _GdkGeometry           GdkGeometry;
-typedef struct _GdkWindowAttr	      GdkWindowAttr;
-typedef struct _GdkPointerHooks	      GdkPointerHooks;
+typedef struct _GdkGeometry          GdkGeometry;
+typedef struct _GdkWindowAttr        GdkWindowAttr;
+typedef struct _GdkPointerHooks      GdkPointerHooks;
+typedef struct _GdkWindowRedirect    GdkWindowRedirect;
 
 /* Classes of windows.
  *   InputOutput: Almost every window should be of this type. Such windows
@@ -72,7 +77,8 @@ typedef enum
   GDK_WINDOW_CHILD,
   GDK_WINDOW_DIALOG,
   GDK_WINDOW_TEMP,
-  GDK_WINDOW_FOREIGN
+  GDK_WINDOW_FOREIGN,
+  GDK_WINDOW_OFFSCREEN
 } GdkWindowType;
 
 /* Window attribute mask values.
@@ -254,6 +260,12 @@ typedef struct _GdkWindowObjectClass GdkWindowObjectClass;
 #define GDK_WINDOW_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), GDK_TYPE_WINDOW, GdkWindowObjectClass))
 #define GDK_WINDOW_OBJECT(object)    ((GdkWindowObject *) GDK_WINDOW (object))
 
+#ifndef GDK_COMPILATION
+/* We used to export all of GdkWindowObject, but we don't want to keep doing so.
+   However, there are various parts of it accessed by macros and other code,
+   so we keep the old exported version public, but in reality it is larger. */
+
+/**** DON'T CHANGE THIS STRUCT, the real version is in gdkinternals.h ****/
 struct _GdkWindowObject
 {
   GdkDrawable parent_instance;
@@ -298,7 +310,12 @@ struct _GdkWindowObject
   guint shaped : 1;
   
   GdkEventMask event_mask;
+
+  guint update_and_descendants_freeze_count;
+
+  GdkWindowRedirect *redirect;
 };
+#endif
 
 struct _GdkWindowObjectClass
 {
@@ -363,13 +380,14 @@ void          gdk_window_add_filter            (GdkWindow     *window,
 void          gdk_window_remove_filter         (GdkWindow     *window,
                                                 GdkFilterFunc  function,
                                                 gpointer       data);
-void          gdk_window_scroll                (GdkWindow *window,
-                                                gint       dx,
-                                                gint       dy);
-void	      gdk_window_move_region           (GdkWindow *window,
-						GdkRegion *region,
-						gint       dx,
-						gint       dy);
+void          gdk_window_scroll                (GdkWindow     *window,
+                                                gint           dx,
+                                                gint           dy);
+void	      gdk_window_move_region           (GdkWindow       *window,
+						const GdkRegion *region,
+						gint             dx,
+						gint             dy);
+gboolean      gdk_window_ensure_native        (GdkWindow       *window);
 
 /* 
  * This allows for making shaped (partially transparent) windows
@@ -377,14 +395,14 @@ void	      gdk_window_move_region           (GdkWindow *window,
  *  The shape_mask can be the mask
  *  from gdk_pixmap_create_from_xpm.   Stefan Wille
  */
-void gdk_window_shape_combine_mask  (GdkWindow	    *window,
-                                     GdkBitmap	    *mask,
-                                     gint	     x,
-                                     gint	     y);
-void gdk_window_shape_combine_region (GdkWindow	    *window,
-                                      GdkRegion     *shape_region,
-                                      gint	     offset_x,
-                                      gint	     offset_y);
+void gdk_window_shape_combine_mask  (GdkWindow	      *window,
+                                     GdkBitmap	      *mask,
+                                     gint	       x,
+                                     gint	       y);
+void gdk_window_shape_combine_region (GdkWindow	      *window,
+                                      const GdkRegion *shape_region,
+                                      gint	       offset_x,
+                                      gint	       offset_y);
 
 /*
  * This routine allows you to quickly take the shapes of all the child windows
@@ -396,7 +414,7 @@ void gdk_window_shape_combine_region (GdkWindow	    *window,
 void gdk_window_set_child_shapes (GdkWindow *window);
 
 void gdk_window_set_composited   (GdkWindow *window,
-                                  gboolean composited);
+                                  gboolean   composited);
 
 /*
  * This routine allows you to merge (ie ADD) child shapes to your
@@ -405,18 +423,18 @@ void gdk_window_set_composited   (GdkWindow *window,
  * 
  * - Raster
  */
-void gdk_window_merge_child_shapes (GdkWindow *window);
+void gdk_window_merge_child_shapes         (GdkWindow       *window);
 
-void gdk_window_input_shape_combine_mask   (GdkWindow *window,
-					    GdkBitmap *mask,
-					    gint       x,
-					    gint       y);
-void gdk_window_input_shape_combine_region (GdkWindow *window,
-                                            GdkRegion *shape_region,
-                                            gint       offset_x,
-                                            gint       offset_y);
-void gdk_window_set_child_input_shapes     (GdkWindow *window);
-void gdk_window_merge_child_input_shapes   (GdkWindow *window);
+void gdk_window_input_shape_combine_mask   (GdkWindow       *window,
+					    GdkBitmap       *mask,
+					    gint             x,
+					    gint             y);
+void gdk_window_input_shape_combine_region (GdkWindow       *window,
+                                            const GdkRegion *shape_region,
+                                            gint             offset_x,
+                                            gint             offset_y);
+void gdk_window_set_child_input_shapes     (GdkWindow       *window);
+void gdk_window_merge_child_input_shapes   (GdkWindow       *window);
 
 
 /*
@@ -473,16 +491,16 @@ void gdk_window_set_skip_pager_hint   (GdkWindow *window,
 void gdk_window_set_urgency_hint      (GdkWindow *window,
 				       gboolean   urgent);
 
-void          gdk_window_set_geometry_hints (GdkWindow        *window,
-					     GdkGeometry      *geometry,
-					     GdkWindowHints    geom_mask);
-void          gdk_set_sm_client_id         (const gchar *sm_client_id);
+void          gdk_window_set_geometry_hints (GdkWindow          *window,
+					     const GdkGeometry  *geometry,
+					     GdkWindowHints      geom_mask);
+void          gdk_set_sm_client_id          (const gchar        *sm_client_id);
 
-void	      gdk_window_begin_paint_rect   (GdkWindow    *window,
-					     GdkRectangle *rectangle);
-void	      gdk_window_begin_paint_region (GdkWindow    *window,
-					     GdkRegion    *region);
-void	      gdk_window_end_paint          (GdkWindow    *window);
+void	      gdk_window_begin_paint_rect   (GdkWindow          *window,
+					     const GdkRectangle *rectangle);
+void	      gdk_window_begin_paint_region (GdkWindow          *window,
+					     const GdkRegion    *region);
+void	      gdk_window_end_paint          (GdkWindow          *window);
 
 void	      gdk_window_set_title	   (GdkWindow	  *window,
 					    const gchar	  *title);
@@ -513,8 +531,13 @@ void	      gdk_window_get_position	 (GdkWindow	  *window,
 gint	      gdk_window_get_origin	 (GdkWindow	  *window,
 					  gint		  *x,
 					  gint		  *y);
+void	      gdk_window_get_root_coords (GdkWindow	  *window,
+					  gint             x,
+					  gint             y,
+					  gint		  *root_x,
+					  gint		  *root_y);
 
-#if !defined (GDK_DISABLE_DEPRECATED) || defined (GTK_COMPILATION)
+#if !defined (GDK_DISABLE_DEPRECATED) || defined (GTK_COMPILATION) || defined (GDK_COMPILATION)
 /* Used by gtk_handle_box_button_changed () */
 gboolean      gdk_window_get_deskrelative_origin (GdkWindow	  *window,
 					  gint		  *x,
@@ -556,7 +579,7 @@ gboolean      gdk_window_get_decorations (GdkWindow       *window,
 					  GdkWMDecoration *decorations);
 void	      gdk_window_set_functions	 (GdkWindow	  *window,
 					  GdkWMFunction	   functions);
-#ifndef GDK_MULTIHEAD_SAFE
+#if !defined(GDK_MULTIHEAD_SAFE) && !defined(GDK_DISABLE_DEPRECATED)
 GList *       gdk_window_get_toplevels   (void);
 #endif
 
@@ -590,14 +613,14 @@ void gdk_window_begin_move_drag   (GdkWindow     *window,
                                    guint32        timestamp);
 
 /* Interface for dirty-region queueing */
-void       gdk_window_invalidate_rect           (GdkWindow    *window,
-					         GdkRectangle *rect,
-					         gboolean      invalidate_children);
-void       gdk_window_invalidate_region         (GdkWindow    *window,
-					         GdkRegion    *region,
-					         gboolean      invalidate_children);
-void       gdk_window_invalidate_maybe_recurse  (GdkWindow *window,
-						 GdkRegion *region,
+void       gdk_window_invalidate_rect           (GdkWindow          *window,
+					         const GdkRectangle *rect,
+					         gboolean            invalidate_children);
+void       gdk_window_invalidate_region         (GdkWindow          *window,
+					         const GdkRegion    *region,
+					         gboolean            invalidate_children);
+void       gdk_window_invalidate_maybe_recurse  (GdkWindow          *window,
+						 const GdkRegion    *region,
 						 gboolean (*child_func) (GdkWindow *, gpointer),
 						 gpointer   user_data);
 GdkRegion *gdk_window_get_update_area     (GdkWindow    *window);
@@ -605,6 +628,9 @@ GdkRegion *gdk_window_get_update_area     (GdkWindow    *window);
 void       gdk_window_freeze_updates      (GdkWindow    *window);
 void       gdk_window_thaw_updates        (GdkWindow    *window);
 
+void       gdk_window_freeze_toplevel_updates_libgtk_only (GdkWindow *window);
+void       gdk_window_thaw_toplevel_updates_libgtk_only   (GdkWindow *window);
+
 void       gdk_window_process_all_updates (void);
 void       gdk_window_process_updates     (GdkWindow    *window,
 					   gboolean      update_children);
@@ -633,6 +659,23 @@ GdkPointerHooks *gdk_set_pointer_hooks (const GdkPointerHooks *new_hooks);
 
 GdkWindow *gdk_get_default_root_window (void);
 
+/* Offscreen redirection */
+GdkPixmap *gdk_offscreen_window_get_pixmap     (GdkWindow     *window);
+void       gdk_offscreen_window_set_embedder   (GdkWindow     *window,
+						GdkWindow     *embedder);
+GdkWindow *gdk_offscreen_window_get_embedder   (GdkWindow     *window);
+void       gdk_window_geometry_changed         (GdkWindow     *window);
+
+void       gdk_window_redirect_to_drawable   (GdkWindow     *window,
+                                              GdkDrawable   *drawable,
+                                              gint           src_x,
+                                              gint           src_y,
+                                              gint           dest_x,
+                                              gint           dest_y,
+                                              gint           width,
+                                              gint           height);
+void       gdk_window_remove_redirection     (GdkWindow     *window);
+
 #ifndef GDK_DISABLE_DEPRECATED
 #define GDK_ROOT_PARENT()             (gdk_get_default_root_window ())
 #define gdk_window_get_size            gdk_drawable_get_size
